# -*- coding: utf-8 -*-
#
import math
import mod.server.extraServerApi as serverApi
import random

CompFactory = serverApi.GetEngineCompFactory()


class SpawnMonsterServerSystem(serverApi.GetServerSystemCls()):
    def __init__(self, namespace, name):
        super(SpawnMonsterServerSystem, self).__init__(namespace, name)
        self.ListenEvent()
        # 组件缓存，避免重复创建
        self.mTimeComp = CompFactory.CreateTime(serverApi.GetLevelId())
        #

    def ListenEvent(self):
        self.ListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(), "ServerSpawnMobEvent", self,
                            self.OnServerSpawnMobEvent)

    def UnListenEvent(self):
        self.UnListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(), "ServerSpawnMobEvent", self,
                              self.OnServerSpawnMobEvent)

    def Destroy(self):
        self.UnListenEvent()

    # region 监听事件
    # --------------------------------------------------------------------------------------------
    # 游戏内自动生成生物
    def OnServerSpawnMobEvent(self, args):
        entityId = args['entityId']
        identifier = args['realIdentifier']
        # if identifier != 'minecraft:cow':
        #     args['cancel'] = True
        # else:
        #     print '====生成了实体：', args['x'], args['y'], args['z']

    # endregion

    # region 类函数
    # --------------------------------------------------------------------------------------------
    # 在玩家的周围生成怪物
    def SpawnMonsterNearPlayer(self):
        pass

    def SpawnEntityAround(self, targetId, spawnEntityIdentifier, num=1, minRadius=6, maxRadius=12):
        """
        生成实体在目标周围
        :param targetId:
        :param spawnEntityIdentifier:
        :param num:
        :param minRadius:
        :param maxRadius:
        :return:
        """
        entityList = []

        targetX, targetY, targetZ = CompFactory.CreatePos(targetId).GetFootPos()
        dimensionId = CompFactory.CreateDimension(targetId).GetEntityDimensionId()

        for _num in xrange(num):
            _randomPos = (
                targetX + random.randint(minRadius, maxRadius) * math.sin(random.uniform(0, 2 * math.pi)),
                targetY,
                targetZ + random.randint(minRadius, maxRadius) * math.cos(random.uniform(0, 2 * math.pi))
            )
            # 上面的 _randomPos 只是平面上的一个坐标，我们为了合理生成在地面上，需要 y 在 [-20, 20] 之间有一个合适的坐标
            _firstAirPos = self._RetFirstAirBlockPos(_randomPos, dimensionId, -20, 20)
            if _firstAirPos:
                entityId = self.CreateEngineEntityByTypeStr(spawnEntityIdentifier, _firstAirPos, (0, random.randint(-180, 180)),
                                                            dimensionId)
                entityList.append(entityId)
        return entityList

    # 范围内的第一个空气方块
    def _RetFirstAirBlockPos(self, basePos, dimensionId, step, maxStep):
        blockInfoComp = CompFactory.CreateBlockInfo(serverApi.GetLevelId())
        while step < maxStep:
            finalPos = (basePos[0], basePos[1] + step, basePos[2])
            blockDict = blockInfoComp.GetBlockNew(finalPos, dimensionId)
            step += 1
            if blockDict and blockDict['name'] == 'minecraft:air':
                return finalPos
        return None

    # endregion
