# -*- coding: utf-8 -*-
import mod.client.extraClientApi as clientApi

CompFactory = clientApi.GetEngineCompFactory()


class TimeRuleClientSystem(clientApi.GetClientSystemCls()):

    def __init__(self, namespace, name):
        super(TimeRuleClientSystem, self).__init__(namespace, name)
        self.ListenEvent()
        self.mTimeCounter = 0
        self.mUINode = None
        #
        self.mSkyRenderComp = CompFactory.CreateSkyRender(clientApi.GetLevelId())
        self.mQueryVariableComp = CompFactory.CreateQueryVariable(clientApi.GetLocalPlayerId())
        self.mFogComp = CompFactory.CreateFog(clientApi.GetLevelId())

    def ListenEvent(self):
        # 自定义事件
        self.ListenForEvent("timeRuleMod", "timeRuleServerSystem", "BloodMoonEvent", self, self.OnBloodMoonEvent)
        self.ListenForEvent("timeRuleMod", "timeRuleServerSystem", "FoggyWeatherEvent", self, self.OnFoggyWeatherEvent)
        # 系统事件
        self.ListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "UiInitFinished",
                            self, self.OnUiInitFinished)
        self.ListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "AddPlayerCreatedClientEvent",
                            self, self.OnAddPlayerCreatedClientEvent)
        self.ListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "OnKeyPressInGame",
                            self, self.OnOnKeyPressInGame)

    def UnListenEVent(self):
        # 自定义事件
        self.UnListenForEvent("timeRuleMod", "timeRuleServerSystem", "BloodMoonEvent", self, self.OnBloodMoonEvent)
        self.UnListenForEvent("timeRuleMod", "timeRuleServerSystem", "FoggyWeatherEvent", self, self.OnFoggyWeatherEvent)
        # 系统事件
        self.UnListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "UiInitFinished",
                              self, self.OnUiInitFinished)
        self.UnListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "AddPlayerCreatedClientEvent",
                              self, self.OnAddPlayerCreatedClientEvent)
        self.UnListenForEvent(clientApi.GetEngineNamespace(), clientApi.GetEngineSystemName(), "OnKeyPressInGame",
                              self, self.OnOnKeyPressInGame)

    def Destroy(self):
        self.UnListenEVent()

    def Update(self):
        self.mTimeCounter += 1
        if self.mTimeCounter % 3 == 0:
            moonRot = self.mSkyRenderComp.GetMoonRot()[2]
            self.mQueryVariableComp.Set('query.mod.moon_rot', moonRot)

    def OnBloodMoonEvent(self, args):
        flag = args['flag']
        self.mQueryVariableComp.Set('query.mod.play_blood_moon', 1 if flag else 0)

    def OnFoggyWeatherEvent(self, args):
        flag = args['flag']
        self._FoggyWeather(flag)

    def OnUiInitFinished(self, args=None):
        # 注册 UI
        uiClsPath = 'timeRuleScripts.uIScripts.UIScript'
        uiScreenDef = 'timeDisplayUI.main'
        clientApi.RegisterUI('timeRuleMod', 'timeDisplayUI', uiClsPath, uiScreenDef)
        # 创建 UI
        self.mUINode = clientApi.CreateUI('timeRuleMod', 'timeDisplayUI', {"isHud": 1})
        if self.mUINode:
            self.mUINode.Init()  # 调用初始化函数

    def OnAddPlayerCreatedClientEvent(self, args):
        playerId = args['playerId']
        if playerId == clientApi.GetLocalPlayerId():
            self.InitRender()

    def OnOnKeyPressInGame(self, args):
        screenName = args['screenName']
        key = int(args['key'])
        isDown = int(args['isDown'])
        if isDown:
            if key == clientApi.GetMinecraftEnum().KeyBoardType.KEY_TAB:
                self.NotifyToServer("SwitchGameMode", {"playerId": clientApi.GetLocalPlayerId()})
            elif key == clientApi.GetMinecraftEnum().KeyBoardType.KEY_I:
                comp = clientApi.GetEngineCompFactory().CreateSkyRender(clientApi.GetLevelId())
                comp.ResetSkyColor()

    # 初始化绑定，给玩家绑定上血月的相关 query.mod.xx 变量和相关动画
    def InitRender(self):
        queryVariableComp = CompFactory.CreateQueryVariable(clientApi.GetLocalPlayerId())
        queryVariableComp.Register("query.mod.moon_rot", 0)
        queryVariableComp.Register("query.mod.play_blood_moon", 0)

        actorRenderComp = CompFactory.CreateActorRender(clientApi.GetLocalPlayerId())
        actorRenderComp.AddPlayerParticleEffect("blood_moon", "tutorial:blood_moon")
        actorRenderComp.AddPlayerAnimation("play_blood_moon", "animation.blood_moon.play")
        actorRenderComp.AddPlayerScriptAnimate("play_blood_moon", "query.mod.play_blood_moon")
        res = actorRenderComp.RebuildPlayerRender()

    def _FoggyWeather(self, isOpen):
        if isOpen:
            self.mFogComp.SetFogLength(0.1, 32)
        else:
            self.mFogComp.ResetFogLength()
