# -*- coding: utf-8 -*-
#
import mod.server.extraServerApi as serverApi
import random

CompFactory = serverApi.GetEngineCompFactory()

# 僵尸的标识符
ZombieIdentifier = 'minecraft:zombie'
# 僵尸的等级与天数的对应关系
ZombieLevelDay = {
    1 : 1,
    10: 2,
    20: 3
}
# 僵尸等级与僵尸属性的对应关系
ZombieLevelAttr = {
    1: {
        serverApi.GetMinecraftEnum().AttrType.HEALTH: 10,
        serverApi.GetMinecraftEnum().AttrType.SPEED : 0.25,
        serverApi.GetMinecraftEnum().AttrType.DAMAGE: 1
    },
    2: {
        serverApi.GetMinecraftEnum().AttrType.HEALTH: 20,
        serverApi.GetMinecraftEnum().AttrType.SPEED : 0.3,
        serverApi.GetMinecraftEnum().AttrType.DAMAGE: 2
    },
    3: {
        serverApi.GetMinecraftEnum().AttrType.HEALTH: 30,
        serverApi.GetMinecraftEnum().AttrType.SPEED : 0.4,
        serverApi.GetMinecraftEnum().AttrType.DAMAGE: 3
    }
}
# 僵尸的数量限制
ZombieLimit = 100


class TimeRuleServerSystem(serverApi.GetServerSystemCls()):
    def __init__(self, namespace, name):
        super(TimeRuleServerSystem, self).__init__(namespace, name)
        self.ListenEvent()
        # 组件缓存，避免重复创建
        self.mTimeComp = CompFactory.CreateTime(serverApi.GetLevelId())
        #
        self.mTimeCounter = 0
        self.mLastDay = 0  # 上一次的天数，用来判断是否是新的一天
        self.mCurrentDay = 0  # 今天的天数
        #
        self.mTicksPerDay = 24000  # 一天的刻数
        self.mCurrentTime = 0  # 当前的时间
        self.mCurrentTimeIsDayTime = True  # 当前的时间是否是白天
        #
        self.mIsBloodMoon = False  # 是否开启了血月
        self.mIsFoggyWeather = False  # 是否开启了大雾天气

    def Update(self):
        self.mTimeCounter += 1
        # 每一秒都检查
        if self.mTimeCounter % 30 == 0:
            self._TriggerZombieEventIfNewDay()
            self._TriggerRandomEventIfNewDayOrNight()
            # 血月的时候会检测生成更多的实体
            if not self.mCurrentTimeIsDayTime and self.mIsBloodMoon:
                self.SpawnZombieAroundPlayer()

        currentTime = self.mTimeComp.GetTime()
        # self.mTimeComp.SetTime(currentTime + 20)

    def ListenEvent(self):
        self.ListenForEvent('timeRuleMod', 'timeRuleClientSystem', "SwitchGameMode", self, self.OnSwitchGameMode)
        self.ListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(), "AddEntityServerEvent", self,
                            self.OnAddEntityServerEvent)

    def UnListenEvent(self):
        self.UnListenForEvent('timeRuleMod', 'timeRuleClientSystem', "SwitchGameMode", self, self.OnSwitchGameMode)
        self.UnListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(), "AddEntityServerEvent", self,
                              self.OnAddEntityServerEvent)

    def Destroy(self):
        self.UnListenEvent()

    # region 监听事件
    # --------------------------------------------------------------------------------------------
    def OnSwitchGameMode(self, args):
        playerId = args['playerId']
        playerComp = CompFactory.CreatePlayer(playerId)
        gameType = CompFactory.CreateGame(serverApi.GetLevelId()).GetPlayerGameType(playerId)
        if gameType != serverApi.GetMinecraftEnum().GameType.Survival:
            setGameType = serverApi.GetMinecraftEnum().GameType.Survival
        else:
            setGameType = serverApi.GetMinecraftEnum().GameType.Creative
        playerComp.SetPlayerGameType(setGameType)

    def OnAddEntityServerEvent(self, args):
        entityId = args['id']
        engineTypeStr = args['engineTypeStr']
        if engineTypeStr == ZombieIdentifier:
            self._SetEntityLevelAndAttrAccordCurrentDay(entityId)

    # endregion

    # region 类函数
    # --------------------------------------------------------------------------------------------
    def _TriggerZombieEventIfNewDay(self):
        # 从游戏开始经过的总帧数
        passedTime = self.mTimeComp.GetTime()
        # 从游戏开始经过的游戏天数
        day = passedTime / self.mTicksPerDay
        if day != self.mCurrentDay:
            self.mCurrentDay = day
        isNewDay = self.mLastDay != self.mCurrentDay
        self.mLastDay = self.mCurrentDay

        if isNewDay:
            self._ResetAllZombieLevel()

    # 重置所有僵尸的等级
    def _ResetAllZombieLevel(self):
        print '=====重置所有僵尸等级===='
        for entityId, entityDict in serverApi.GetEngineActor().items():
            for _dimensionId, _identifier in entityDict.items():
                if _identifier == ZombieIdentifier:
                    self._SetEntityLevelAndAttrAccordCurrentDay(entityId)

    def _SetEntityLevelAndAttrAccordCurrentDay(self, entityId):
        # 使用事件来触发对应僵尸的等级的行为
        zombieLevel = self._GetZombieLevelByDay(self.mCurrentDay)
        eventName = 'tutorial:zombie_level_' + str(zombieLevel)
        CompFactory.CreateEntityEvent(entityId).TriggerCustomEvent(entityId, eventName)
        # 根据僵尸等级来设置僵尸的属性值
        attrComp = CompFactory.CreateAttr(entityId)
        for _attrName, _attrValue in ZombieLevelAttr[zombieLevel].items():
            attrComp.SetAttrMaxValue(_attrName, _attrValue)
            attrComp.SetAttrValue(_attrName, _attrValue)

    # 根据天数来获取当前僵尸等级
    def _GetZombieLevelByDay(self, day):
        for threshold, level in sorted(ZombieLevelDay.items(), reverse=True):
            if day >= threshold:
                return level
        return 1  # 默认等级为1

    # 在新的白天或者夜晚触发随机事件
    def _TriggerRandomEventIfNewDayOrNight(self):
        # 从游戏开始经过的总帧数
        passedTime = self.mTimeComp.GetTime()
        previousTick = self.mCurrentTime
        self.mCurrentTime = passedTime

        # 计算当前时间位置
        currentTime = self.mCurrentTime % self.mTicksPerDay
        previousTime = previousTick % self.mTicksPerDay

        # 检查是否发生了昼夜交替
        if currentTime < (self.mTicksPerDay / 2) <= previousTime:
            self._HandleNightToDayTransition()
        elif currentTime >= (self.mTicksPerDay / 2) > previousTime:
            self._HandleDayToNightTransition()

        # 更新白天/夜晚状态
        self.mCurrentTimeIsDayTime = currentTime < (self.mTicksPerDay / 2)

    def _HandleDayToNightTransition(self):
        self._CloseDayEvent()
        self._TriggerNightEvent()

    def _CloseDayEvent(self):
        if self.mIsFoggyWeather:
            # 让客户端关闭大雾天气渲染
            self.BroadcastToAllClient("FoggyWeatherEvent", {'flag': False})
            self.mIsFoggyWeather = False

    def _TriggerNightEvent(self):
        if random.random() < self._GetEventProbability():
            # 通知全部玩家
            CompFactory.CreateCommand(serverApi.GetLevelId()).SetCommand("/title @a title §c血月降临!!")
            self.mIsBloodMoon = True
            # 让客户端渲染血月
            self.BroadcastToAllClient("BloodMoonEvent", {'flag': True})
            # 在开始的 10 秒内，不断在玩家周围生成僵尸

    def _HandleNightToDayTransition(self):
        self._CloseNightEvent()
        self._TriggerDayEvent()

    def _CloseNightEvent(self):
        if self.mIsBloodMoon:
            # 让客户端关闭血月渲染
            self.BroadcastToAllClient("BloodMoonEvent", {'flag': False})
            self.mIsBloodMoon = False

    def _TriggerDayEvent(self):
        if random.random() < self._GetEventProbability():
            CompFactory.CreateCommand(serverApi.GetLevelId()).SetCommand("/title @a title 大雾天气!!")
            # 通知客户端渲染大雾天气
            self.BroadcastToAllClient("FoggyWeatherEvent", {'flag': True})
            self.mIsFoggyWeather = True

    # 触发事件的概率。随天数的增加，概率逐渐增高，最多为 0.9，最少为 0.1
    def _GetEventProbability(self):
        return min(0.9, (0.1 + (self.mCurrentDay / 100.0)))

    # endregion

    # region 生成怪物
    def SpawnZombieAroundPlayer(self):
        # 需要先检查当前世界上的僵尸数量
        if self.CurrentZombieNum() <= ZombieLimit:
            randomPlayerId = random.choice(serverApi.GetPlayerList())
            self.SpawnEntityAround(randomPlayerId, ZombieIdentifier, 10)

    def CurrentZombieNum(self):
        count = 0
        for entityId, entityDict in serverApi.GetEngineActor().items():
            for _dimensionId, _identifier in entityDict.items():
                if _identifier == ZombieIdentifier:
                    count += 1
        return count

    def SpawnEntityAround(self, targetId, spawnEntityIdentifier, num=1, minRadius=6, maxRadius=12):
        """
        生成实体在目标周围
        :param targetId:
        :param spawnEntityIdentifier:
        :param num:
        :param minRadius:
        :param maxRadius:
        :return:
        """
        entityList = []

        targetX, targetY, targetZ = CompFactory.CreatePos(targetId).GetFootPos()
        dimensionId = CompFactory.CreateDimension(targetId).GetEntityDimensionId()

        for _num in xrange(num):
            _randomPos = (
                targetX + random.randint(minRadius, maxRadius) * math.sin(random.uniform(0, 2 * math.pi)),
                targetY,
                targetZ + random.randint(minRadius, maxRadius) * math.cos(random.uniform(0, 2 * math.pi))
            )
            # 上面的 _randomPos 只是平面上的一个坐标，我们为了合理生成在地面上，需要 y 在 [-20, 20] 之间有一个合适的坐标
            _firstAirPos = self._RetFirstAirBlockPos(_randomPos, dimensionId, -20, 20)
            if _firstAirPos:
                entityId = self.CreateEngineEntityByTypeStr(spawnEntityIdentifier, _firstAirPos, (0, random.randint(-180, 180)),
                                                            dimensionId)
                entityList.append(entityId)
        return entityList

    # 范围内的第一个空气方块
    def _RetFirstAirBlockPos(self, basePos, dimensionId, step, maxStep):
        blockInfoComp = CompFactory.CreateBlockInfo(serverApi.GetLevelId())
        while step < maxStep:
            finalPos = (basePos[0], basePos[1] + step, basePos[2])
            blockDict = blockInfoComp.GetBlockNew(finalPos, dimensionId)
            step += 1
            if blockDict and blockDict['name'] == 'minecraft:air':
                return finalPos
        return None

    # endregion
