# -*- coding: utf-8 -*-
import mod.client.extraClientApi as clientApi

ScreenNode = clientApi.GetScreenNodeCls()

CompFactory = clientApi.GetEngineCompFactory()
gameComp = CompFactory.CreateGame(clientApi.GetLevelId())


class UIScript(ScreenNode):
    def __init__(self, namespace, name, param):
        ScreenNode.__init__(self, namespace, name, param)
        self.mPlayerId = clientApi.GetLocalPlayerId()

        # 组件注册地址
        self.mLabelPath = '/label'

        # 界面需要使用的自定义属性
        self.mTimeCounter = 0

    def Create(self):
        print("===== UI Create =====")

    # 1 秒 30 帧
    def Update(self):
        self.mTimeCounter += 1
        perSec = self.mTimeCounter % 30 == 0
        if perSec:
            self.UpdateLabelContent()

    # region 类函数
    # --------------------------------------------------------------------------------------------
    def Init(self):
        print '=== UI 初始化 ==='
        self.UpdateLabelContent()

    def UpdateLabelContent(self):
        timeComp = CompFactory.CreateTime(clientApi.GetLevelId())
        pressedTime = timeComp.GetTime()
        timeStr = self.GameTime2RealTime(pressedTime)
        self.GetLabel(self.mLabelPath).SetText(timeStr)

    def GameTime2RealTime(self, gameTick):
        # 定义游戏中一天的刻数
        gameDayTicks = 24000
        # 定义游戏中一小时的刻数
        gameHourTicks = gameDayTicks / 24
        # 定义游戏中一分钟的刻数
        gameMinuteTicks = gameHourTicks / 60
        # 计算游戏中的天数
        gameDay = gameTick // gameDayTicks + 1
        # 计算游戏中的小时数
        gameHour = (gameTick % gameDayTicks) // gameHourTicks
        # 计算游戏中的分钟数
        gameMinute = (gameTick % gameHourTicks) // gameMinuteTicks
        # 把游戏中的小时数转换成现实中的小时数，加上6小时的偏移量
        realHour = (gameHour + 6) % 24
        # 把现实中的小时数、分钟数转换成字符串，补齐两位
        realHourStr = str(realHour).zfill(2)
        realMinuteStr = str(gameMinute).zfill(2)
        # 返回转换后的格式
        return "第{}天第{}时第{}分".format(gameDay, realHourStr, realMinuteStr)

    def GetLabel(self, path):
        control = self.GetBaseUIControl(path)
        if control:
            return control.asLabel()
        return None

    # endregion
